@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -33,8 +33,8 @@
 
 #include "qcosmeticstroker_p.h"
 #include "private/qpainterpath_p.h"
+#include "private/qrgba64_p.h"
 #include <qdebug.h>
-#include <math.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -281,7 +281,7 @@ void QCosmeticStroker::setup()
     drawCaps = state->lastPen.capStyle() != Qt::FlatCap;
 
     if (strokeSelection & FastDraw) {
-        color = INTERPOLATE_PIXEL_256(state->penData.solid.color, opacity, 0, 0);
+        color = multiplyAlpha256(state->penData.solid.color, opacity).toArgb32();
         QRasterBuffer *buffer = state->penData.rasterBuffer;
         pixels = (uint *)buffer->buffer();
         ppl = buffer->bytesPerLine()>>2;
@@ -438,8 +438,9 @@ void QCosmeticStroker::calculateLastPoint(qreal rx1, qreal ry1, qreal rx2, qreal
         int y = (y1 + 32) >> 6;
         int ys = (y2 + 32) >> 6;
 
+        int round = (xinc > 0) ? 32 : 0;
         if (y != ys) {
-            x += ( ((((y << 6) + 32 - y1)))  * xinc ) >> 6;
+            x += ( ((((y << 6) + round - y1)))  * xinc ) >> 6;
 
             if (swapped) {
                 lastPixel.x = x >> 16;
@@ -469,8 +470,9 @@ void QCosmeticStroker::calculateLastPoint(qreal rx1, qreal ry1, qreal rx2, qreal
         int x = (x1 + 32) >> 6;
         int xs = (x2 + 32) >> 6;
 
+        int round = (yinc > 0) ? 32 : 0;
         if (x != xs) {
-            y += ( ((((x << 6) + 32 - x1)))  * yinc ) >> 6;
+            y += ( ((((x << 6) + round - x1)))  * yinc ) >> 6;
 
             if (swapped) {
                 lastPixel.x = x;
@@ -718,10 +720,11 @@ static inline void capAdjust(int caps, int &x1, int &x2, int &y, int yinc)
 template<DrawPixel drawPixel, class Dasher>
 static bool drawLine(QCosmeticStroker *stroker, qreal rx1, qreal ry1, qreal rx2, qreal ry2, int caps)
 {
+    bool didDraw = qAbs(rx2 - rx1) + qAbs(ry2 - ry1) >= 1.0;
+
     if (stroker->clipLine(rx1, ry1, rx2, ry2))
         return true;
 
-    bool didDraw = false;
     const int half = stroker->legacyRounding ? 31 : 0;
     int x1 = toF26Dot6(rx1) + half;
     int y1 = toF26Dot6(ry1) + half;
@@ -757,9 +760,10 @@ static bool drawLine(QCosmeticStroker *stroker, qreal rx1, qreal ry1, qreal rx2,
 
         int y = (y1 + 32) >> 6;
         int ys = (y2 + 32) >> 6;
+        int round = (xinc > 0) ? 32 : 0;
 
         if (y != ys) {
-            x += ( ((((y << 6) + 32 - y1)))  * xinc ) >> 6;
+            x += ( ((((y << 6) + round - y1)))  * xinc ) >> 6;
 
             // calculate first and last pixel and perform dropout control
             QCosmeticStroker::Point first;
@@ -833,9 +837,10 @@ static bool drawLine(QCosmeticStroker *stroker, qreal rx1, qreal ry1, qreal rx2,
 
         int x = (x1 + 32) >> 6;
         int xs = (x2 + 32) >> 6;
+        int round = (yinc > 0) ? 32 : 0;
 
         if (x != xs) {
-            y += ( ((((x << 6) + 32 - x1)))  * yinc ) >> 6;
+            y += ( ((((x << 6) + round - x1)))  * yinc ) >> 6;
 
             // calculate first and last pixel to perform dropout control
             QCosmeticStroker::Point first;
